import requests
from PIL import Image
from io import BytesIO
import pyquery
import pandas as pd
import numpy as np
import fsia.usgsgatherer.USGSFloodCriteria as criteria
import cv2
import re
import time
import json
import http
import urllib
from fsia.usgsgatherer.imageProcessor import imageProcessor

class imageWaterWatchModel():
    def __init__(self):
        self.id = ""
        self.link = ""

    def setLink(self, link):
        self.link = link
        return self
    def setID(self, id):
        self.id = id
        return self

class usgsFloodManager():
    def __init__(self):
        self.cameras = []
        rocky_creed_cam = imageWaterWatchModel().setLink("http://b7b.hdrelay.com/cameras/fa96bb1e-426d-4b40-a820-251713325420/GetOneShot?size=800x450").setID("021603273")
        lake_moultrie_cam = imageWaterWatchModel().setLink("http://b7b.hdrelay.com/cameras/dab6dc0d-d702-4146-899f-85a7945de140/GetOneShot?size=800x450").setID("02172002")
        rocky_branch_Whalet_cam = imageWaterWatchModel().setLink("http://b7b.hdrelay.com/cameras/2fb4ae88-446d-4632-a849-d426240ccca5/GetOneShot?size=800x450").setID("02169506")
        peedee_river_cam = imageWaterWatchModel().setLink("http://b6b.hdrelay.com/cameras/42336a6b-5be8-443c-97c9-3d70da459e88/GetOneShot?size=800x450").setID("02130810")
        lake_moultrie_trail_canell_cam = imageWaterWatchModel().setLink("http://b7b.hdrelay.com/cameras/dab6dc0d-d702-4146-899f-85a7945de140/GetOneShot?size=800x450").setID("02172002")
        tearcoat_branch_upstream_cam = imageWaterWatchModel().setLink("http://b6b.hdrelay.com/cameras/403818bc-9e8d-4857-9eb0-a675c9668782/GetOneShot?size=800x450").setID("021355015_1")
        tearcoat_branch_downstream_cam = imageWaterWatchModel().setLink("http://b6b.hdrelay.com/cameras/009e9855-a077-4c2a-8289-10eea8ba9f0f/GetOneShot?size=800x450").setID("021355015_2")
        pocotaligo_river_upstream_cam = imageWaterWatchModel().setLink("http://b6b.hdrelay.com/cameras/a8a00672-f089-4b2d-bf36-85e41a838488/GetOneShot?size=800x450").setID("02135615_1")
        pocotaligo_river_downstream_cam = imageWaterWatchModel().setLink("http://b6b.hdrelay.com/cameras/39f23793-0e60-43f6-a0b4-ccb073004640/GetOneShot?size=800x450").setID("02172002_1")
        self.cameras.append(rocky_creed_cam)
        self.cameras.append(lake_moultrie_cam)
        self.cameras.append(rocky_branch_Whalet_cam)
        self.cameras.append(peedee_river_cam)
        self.cameras.append(lake_moultrie_trail_canell_cam)
        self.cameras.append(tearcoat_branch_upstream_cam)
        self.cameras.append(tearcoat_branch_downstream_cam)
        self.cameras.append(pocotaligo_river_upstream_cam)
        self.cameras.append(pocotaligo_river_downstream_cam)

    def getRealTimeWaterWatch(self):
        try:
            usgs_csv = pd.read_csv("https://waterwatch.usgs.gov/webservices/realtime?region=sc&format=csv")
            usgs_df = pd.DataFrame(usgs_csv)
            usgs_df = usgs_df[usgs_df.flow != 0]
            return usgs_df
        except:
            time.sleep(5)
    def getImageWaterWatch(self, id):
        selected_cam = imageWaterWatchModel()
        for cam in self.cameras:
            if cam.id == id:
                selected_cam = cam
        if selected_cam.id == "":
            print("ERROR, camera do not exist")
            return False
        else:
            response = requests.get(selected_cam.link)
            img = Image.open(BytesIO(response.content))
            open_cv_image = np.array(img)
            open_cv_image = open_cv_image[:, :, ::-1].copy()
            image_show = (open_cv_image)
            img_gray = cv2.cvtColor(open_cv_image, cv2.COLOR_BGR2GRAY)
            cv2.imshow('image',img_gray)
            cv2.waitKey(0)
            cv2.destroyAllWindows()

    def getFloodDataCSV(self, USGS_criteria):
        url = "https://waterdata.usgs.gov/"
        url_criteria = ''
        if hasattr(USGS_criteria, 'region'):
            url_criteria += USGS_criteria.region + '/nwis/uv?'
        else:
            url_criteria += 'sc/nwis/'

        if hasattr(USGS_criteria, 'parameters'):
            for parameter in USGS_criteria.parameters:
                url_criteria += 'cb_' + parameter + '=on&'

        url_criteria += 'format=rdb&'

        if hasattr(USGS_criteria, 'station_number'):
            url_criteria += 'site_no='+ USGS_criteria.station_number+ '&'

        if hasattr(USGS_criteria, 'since') and hasattr(USGS_criteria, 'until'):
            url_criteria += 'period=&begin_date=' + str(USGS_criteria.since) + '&end_date=' + str(USGS_criteria.until)

        url += url_criteria
        text = urllib.request.urlopen(url)
        f = open("flood_old.csv", "w+")
        for line in text:
            line = line.decode('utf-8')
            if line[0] == '#':
                pass
            else:
                temp = line.replace('\t', ',')
                f.writelines(temp)
        f.close()



#   Tweet Manager, This class in charge Searching through twitter tweets. The function minic the search 
#   request in browser. The data getting from the tweet is id, source, username, text, created_at, retweet_count
#   favorite_count, mentions, hashtags, geo location, author_id, media 

import urllib.request, urllib.parse, urllib.error,urllib.request,urllib.error,urllib.parse,json,re,datetime,sys,http.cookiejar,time
from pyquery import PyQuery
class TweetModel:
    def __init__(self):
        pass

class TweetManager:

    def __init__(self):
        pass

    @staticmethod
    def getJsonResponse(tweet_criteria, refresh_cursor, cookie_jar, proxy):
        url = "https://twitter.com/i/search/timeline?f=tweets&q=%s&src=typd&%smax_position=%s"
        
        url_get_data = ''
        if hasattr(tweet_criteria, 'username'):
            url_get_data += ' from:' + tweet_criteria.username
        
        if hasattr(tweet_criteria, 'since'):
            url_get_data += ' since:' + tweet_criteria.since

        if hasattr(tweet_criteria, 'until'):
            url_get_data += ' until:' + tweet_criteria.until
            
        if hasattr(tweet_criteria, 'querySearch'):
            url_get_data += ' ' + tweet_criteria.querySearch
            
        if hasattr(tweet_criteria, 'lang'):
            url_lang = 'lang=' + tweet_criteria.lang + '&'
        else:
            url_lang = ''
        url = url % (urllib.parse.quote(url_get_data), url_lang, refresh_cursor)

        headers = [
            ('Host', "twitter.com"),
            ('User-Agent', "Mozilla/5.0 (Windows NT 6.1; Win64; x64)"),
            ('Accept', "application/json, text/javascript, */*; q=0.01"),
            ('Accept-Language', "de,en-US;q=0.7,en;q=0.3"),
            ('X-Requested-With', "XMLHttpRequest"),
            ('Referer', url),
            ('Connection', "keep-alive")
        ]

        if proxy:
            opener = urllib.request.build_opener(urllib.request.ProxyHandler({'http': proxy, 'https': proxy}), urllib.request.HTTPCookieProcessor(cookie_jar))
        else:
            opener = urllib.request.build_opener(urllib.request.HTTPCookieProcessor(cookie_jar))
        opener.addheaders = headers

        try:
            response = opener.open(url)
            json_response = response.read()
        except:
            print("https://twitter.com/search?q=%s&src=typd" % urllib.parse.quote(url_get_data))
            print("Unexpected error:", sys.exc_info()[0])
            sys.exit()
            return
        jason_data = json.loads(json_response.decode())
        return jason_data
    
    @staticmethod
    def getTweets(tweet_criteria, receive_buffer=None, buffer_length=100, proxy=None):
        refresh_cursor = ''

        results = []
        results_aux = []
        cookie_jar = http.cookiejar.CookieJar()

        active = True

        while active:
            json = TweetManager.getJsonResponse(tweet_criteria, refresh_cursor, cookie_jar, proxy)
            
            if len(json['items_html'].strip()) == 0:
                break
            
            refresh_cursor = json['min_position']
            scraped_tweets = PyQuery(json['items_html'])
            scraped_tweets.remove('div.withheld-tweet')
            tweets = scraped_tweets('div.js-stream-tweet')

            if len(tweets) == 0:
                break
            for tweet_HTML in tweets:
                tweet_PQ = PyQuery(tweet_HTML)
                tweet = TweetModel()

                username = tweet_PQ.attr("data-screen-name")
                text = re.sub(r"\s+", " ", tweet_PQ("p.js-tweet-text").text().replace('# ', '#').replace('@ ', '@'))
                retweets = int(tweet_PQ("span.ProfileTweet-action--retweet span.ProfileTweet-actionCount").attr("data-tweet-stat-count").replace(",", ""))
                likes = int(tweet_PQ("span.ProfileTweet-action--favorite span.ProfileTweet-actionCount").attr("data-tweet-stat-count").replace(",", ""))
                date = int(tweet_PQ("small.time span.js-short-timestamp").attr("data-time"))
                id = tweet_PQ.attr("data-tweet-id")
                permalink = tweet_PQ.attr("data-permalink-path")
                user_id = int(tweet_PQ("a.js-user-profile-link").attr("data-user-id"))
                media = tweet_PQ("div.AdaptiveMedia-photoContainer").attr("data-image-url")
                geo = ''
                geo_span = tweet_PQ('span.Tweet-geo')
                if len(geo_span) > 0:
                    geo = geo_span.attr('titile')
                urls = []
                for link in tweet_PQ("a"):
                    try:
                        urls.append((link.attrib["data-expanded-url"]))
                    except KeyError:
                        pass
                tweet.id = id
                tweet.source = 'https://twitter.com' + permalink
                tweet.username = username
                tweet.text = text                
                tweet.created_at = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(date))
                tweet.retweet_count = retweets
                tweet.favorite_count = likes
                tweet.mentions = " ".join(re.compile('(@\\w*)').findall(tweet.text))
                tweet.hashtags = " ".join(re.compile('(#\\w*)').findall(tweet.text))
                tweet.geo = geo
                tweet.urls = ",".join(urls)
                tweet.author_id = user_id
                tweet.media = media
                results.append(tweet)
                results_aux.append(tweet)

                if receive_buffer and len(results_aux) >= buffer_length:
                    receive_buffer(results_aux)
                    results_aux = []

                if tweet_criteria.max_tweets > 0 and len(results) > tweet_criteria.max_tweets:
                    active = False
                    break
        
        if receive_buffer and len(results_aux) > 0:
            receive_buffer(results_aux)

        return results
                







#	Set up the Criteria for USGS Search the Setting that is possible are
#	Username, Time Gap, Query, Language, Max Tweets for data return
#	The Function is then send the Criteria TweetManager.py

class usgsCriteria:
    def __init__(self):
        pass

    def setRegion(self, region):
        self.region = region
        return self

    def setSince(self, since):
        self.since = since
        return self
    
    def setUntil(self, until):
        self.until = until
        return self

    def setStationNumber(self, station_number):
        self.station_number = station_number
        return self
    
    def setParameters(self, parameters):
        self.parameters = parameters
        return self
    




import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from textblob import TextBlob
import re
import sys
import datetime

class tweet_analyzer():
    """
    For analyzing and Categorizing content from tweets    
    """
    def clean_tweet(self,tweet):
        return ' '.join(re.sub("(@[A-Za-z0-9]+)|([^0-9A-Za-z \t])|(\w+:\/\/\S+)", " ", tweet).split())

    def analyze_sentiment(self, tweet):
        analysis = TextBlob(self.clean_tweet(tweet))
        if analysis.sentiment.polarity > 0:
            return 1
        elif analysis.sentiment.polarity == 0:
            return 0
        else:
            return -1

    def tweets_to_dataframe(self, tweets):
        df = pd.DataFrame(data=[tweet.text for tweet in tweets], columns=['Tweets'])
        df['date'] = np.array([tweet.created_at for tweet in tweets])
        df['source'] = np.array([tweet.source for tweet in tweets])
        df['likes'] = np.array([tweet.favorite_count for tweet in tweets])
        df['retweet'] = np.array([tweet.retweet_count for tweet in tweets])        
        df['image_url'] = np.array([tweet.media for tweet in tweets])

        return df



import requests
import fsia.nasagatherer.NasaCredential as NasaCredential
import time
import netCDF4
import os
import cv2
import numpy as np
#from nasa import earth

class SessionWithHeaderRedirection(requests.Session):
    AUTH_HOST = 'urs.earthdata.nasa.gov'
    def __init__(self, username,password):
        super().__init__()        
        self.auth = (username, password)

    def rebuild_auth(self, prepared_request, response):
        headers = prepared_request.headers
        url = prepared_request.url
        if 'Authorization' in headers:
            original_parsed = requests.utils.urlparse(response.request.url)
            redirect_parsed = requests.utils.urlparse(url)      

            if(original_parsed.hostname != redirect_parsed.hostname) and (redirect_parsed.hostname != self.AUTH_HOST) and (original_parsed.hostname != self.AUTH_HOST):
                del headers['Authorization']
        
        return

class NasaGatherer():
    def __init__(self):
        self.username = NasaCredential.NASA_USERNAME
        self.password = NasaCredential.NASA_PASSWORD
        self.session = SessionWithHeaderRedirection(self.username, self.password)
        os.environ["NASA_API_KEY"] = "zqiIogJ4ANsA17OwAvWFYT1ijYHhndlRACafWYKj"

    
    def getNasaEarthData(self,urls):
        for url in urls:
            filename = url[url.rfind('/')+1:]
            try:
                response = self.session.get(url, stream=True)
                with open(filename,'wb') as fd:
                    for chunk in response.iter_content(chunk_size=1024*1024):
                        fd.write(chunk)
            except requests.exceptions.HTTPError as e:
                print(e)
                return
            time.sleep(2)

    def readNasaNetCDF(self):
        rootgrp = netCDF4.Dataset("FLDAS_VIC025_A_EA_D.A20190201.001.nc.nc4", "a" , format="NETCDF4")
        print(rootgrp.file_format)







#	Set up the Criteria for Twitter Search the Setting that is possible are
#	Username, Time Gap, Query, Language, Max Tweets for data return
#	The Function is then send the Criteria TweetManager.py 
class TweetCriteria:
	def __init__(self):
		self.max_tweets = 0

	def setUsername(self, username):
		self.username = username
		return self

	def setSince(self, since):
		self.since = since
		return self

	def setUntil(self, until):
		self.until = until
		return self

	def setQuerySearch(self, querySearch):
		self.querySearch = querySearch
		return self

	def setMaxTweets(self, max_tweets):
		self.max_tweets = max_tweets
		return self

	def setLang(self, Lang):
		self.lang = Lang
		return self

	def setTopTweets(self, topTweets):
 		self.topweets = topTweets
 		return self



import cv2
import numpy as np
class imageProcessor():
    def __init__(self):
        pass
    
    def autoAdustment(self, img):
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        equ = cv2.equalizeHist(gray)
        return equ




import setuptools

with open("README.md" , "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='fsia',
    version='0.1',
    scripts=['fsia'],
    author="Nattapon Donratanapat",
    author_email="pleuk5667@gmail.com",
    description="A data gathering tools for USGS and Twitter",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/VidyaSamadi/Research-Team",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)


from tweepy import API
from tweepy import Cursor
from tweepy.streaming import StreamListener
from tweepy import OAuthHandler
from tweepy import Stream
from tweepy import TweepError
from fsia.tweetgatherer import TweetAnalyzer
import time
import twitterkeys
import pandas as pd

"""
Twitter Client class will authenticate and return twitter client object
"""
class twitterClient():
    def __init__(self, twitter_user_screenname):
        # Authentication Process from Twitter
        self.auth = OAuthHandler(twitterkeys.CONSUMER_KEY, twitterkeys.CONSUMER_SECRET)
        self.auth.set_access_token(twitterkeys.ACCESS_TOKEN, twitterkeys.ACCESS_TOKEN_SECRET)
        self.twitter_client = API(self.auth)
        # Definded Specific User
        self.twitter_user = twitter_user_screenname

    def get_twitter_api(self):
        #return twitter API
        return self.twitter_client

    def get_tweets(self, num_tweet=0):
        #return tweet by the specific user or self
        tweets = []
        if num_tweet == 0:
            tweets = Cursor(self.twitter_client.user_timeline,id=self.twitter_user)
        else:
            for tweet in Cursor(self.twitter_client.user_timeline,id=self.twitter_user).items(num_tweet):
                tweets.append(tweet)
        return tweets

    def get_friend_list(self, num_friends):
        #return Friend list by the specific user or self
        friends = []
        for friend in Cursor(self.twitter_client.friends,id=self.twitter_user).items(num_friends):
            friends.append(friend)
        return friends

    def get_home_tweets(self, num_tweet):
        #return home page tweet list by the specific user or self
        home_tweets = []
        for tweet in Cursor(self.twitter_client.home_timeline,id=self.twitter_user).items(num_tweet):
            home_tweets.append(tweet)
        return home_tweets

    def get_tweets_between_date(self, start, end):
        tweets = []
        c = Cursor(self.twitter_client.user_timeline, id=self.twitter_user).items()
        while True:
            try:
                tweet = c.next()
                if tweet.created_at < end and tweet.created_at > start:
                    tweets.append(tweet)
                if tweet.created_at <= start:
                    return tweets
                time.sleep(0.15)
            except TweepError:
                time.sleep(60*15)
                continue
            except StopIteration:
                return tweets

    def get_tweets_between_date2(self, start, end):
        df = pd.DataFrame()
        c = Cursor(self.twitter_client.user_timeline, id=self.twitter_user).pages()
        analyzer = TweetAnalyzer.tweet_analyzer()
        while True:
            try:
                tweets_list = c.next()
                tweets = []
                for tweet in tweets_list:                
                    if tweet.created_at < end and tweet.created_at > start:
                        tweets.append(tweet)
                    elif tweet.created_at < start:
                        return df
                if len(tweets) != 0:
                    temp = analyzer.tweets_to_dataframe(tweets)
                    if df.empty:
                        df = temp
                    else:                        
                        df.append(temp, ignore_index=True)
                time.sleep(0.15)
            except TweepError:
                time.sleep(60*15)
                continue
            except StopIteration:
                return df
                
    def get_image_tweet(self):
        tweets = []
        c = Cursor(self.twitter_client.user_timeline, id=self.twitter_user, include_entities=True).pages()
        tweets = c.next()
        for tweet in tweets:
            if 'media' in tweet.entities:
                for image in tweet.entities['media']:
                    print(image['media_url'])
                    



     

"""
Class For streaming and processing tweets
"""
class tweetStreamer():
    def __init__(self):
        self.twitter_authenticator = twitterAuthenticator()

    def stream_tweets(self, filename, hashtag_list):
        #handle twitter authentication
        listener = twitterListener(filename)
        auth = self.twitter_authenticator.authenticate_twitter()
        stream = Stream(auth, listener)
        stream.filter(track=hashtag_list)

class twitterAuthenticator():
    """
    Authenticate Class for Tweepy
    """
    def authenticate_twitter(self):
        auth = OAuthHandler(twitterkeys.CONSUMER_KEY, twitterkeys.CONSUMER_SECRET)
        auth.set_access_token(twitterkeys.ACCESS_TOKEN, twitterkeys.ACCESS_TOKEN_SECRET)
        return auth

class twitterListener(StreamListener):
        """
        Basic listener class that just prints received tweets to stdout 
        """
        def __init__(self, fetched_tweets_filename):
            self.fetched_tweets_filename = fetched_tweets_filename
                
        def on_data(self,data):
            try:
                print(data)
                with open(self.fetched_tweets_filename, 'a') as tf:
                    tf.write(data)
                return True 
            except BaseException as e:
                print("error on_data %s" % str(e))
            return True

        def on_error(self, status):
                if status == 420:
                    #return false on_data method in case of rate limit occurs
                    return False
                print(status)        
